bl_info = {
    "name": "Mesh2Rig",
    "author": "Sporenoe3D & Gemini 2.5 Pro",
    "version": (1, 7, 0), # v1.7: Restored bone logic, smaller bones, Added Step 6 Finalize Deform
    "location": "View3D > Sidebar (N-Panel) > Mesh2Rig",
    "description": "Creates rig from mesh density. Merges bones. Adds constraints. Applies pose. Removes Armature Mod. Adds Surface Deform. Bakes action. Finalizes deform. I am not a programmer, but I spent 6 hours working with AI to make this script, completely made up by me",
    "warning": "",
    "doc_url": "",
    "category": "Rigging",
}

import bpy
import bmesh
import mathutils
from mathutils import Vector
from mathutils.kdtree import KDTree
import random
import time # For simple profiling if needed

# --- Helper Functions ---

def get_selected_mesh_object():
    """Returns the first selected MESH object.""" # Translated docstring
    obj = bpy.context.active_object
    if obj and obj.type == 'MESH':
        return obj
    selected_meshes = [o for o in bpy.context.selected_objects if o.type == 'MESH']
    if selected_meshes:
        return selected_meshes[0]
    return None

def get_selected_armature_object():
    """Returns the first selected ARMATURE object.""" # Translated docstring
    obj = bpy.context.active_object
    if obj and obj.type == 'ARMATURE':
        return obj
    selected_armatures = [o for o in bpy.context.selected_objects if o.type == 'ARMATURE']
    if selected_armatures:
        return selected_armatures[0]
    return None

# --- Core Rigging Logic ---

def select_bone_vertices(mesh_obj, target_bone_count, proximity_threshold):
    """
    Selects vertices for bone head placement using proximity (KDTree). # Translated docstring
    (From v1.6 - Generally effective)
    """
    mesh = mesh_obj.data
    if not mesh.vertices:
        print("Mesh has no vertices.")
        return []

    world_matrix = mesh_obj.matrix_world
    vert_coords_world = [world_matrix @ v.co for v in mesh.vertices]
    vert_indices = list(range(len(mesh.vertices))) # Store original indices

    if not vert_coords_world:
        print("Could not get vertex coordinates.")
        return []

    potential_indices = []
    processed_indices = set() # Indices of vertices that are already processed or too close

    # --- KDTree Logic (Proximity Check) ---
    print("Using KDTree proximity check for initial bone placement.")
    if not vert_coords_world: return []
    kdt = KDTree(len(vert_coords_world))
    for i, co in enumerate(vert_coords_world): kdt.insert(co, i)
    kdt.balance()
    effective_threshold = max(proximity_threshold, 0.0001)

    # Iterate through vertices
    indices_to_check = vert_indices
    for i in indices_to_check:
        if i in processed_indices: continue
        potential_indices.append(i)
        processed_indices.add(i)
        # Find neighbors within the threshold and add them to processed set
        nearby_results = kdt.find_range(vert_coords_world[i], effective_threshold)
        nearby_indices = {idx for co, idx, dist_sq in nearby_results if idx != i} # Exclude self
        processed_indices.update(nearby_indices)

    print(f"Found {len(potential_indices)} potential bone locations after proximity filtering.")

    # --- Selecting the Target Number of Bones ---
    if not potential_indices:
        print("No vertices left after proximity filtering.")
        return []

    if len(potential_indices) > target_bone_count:
        print(f"Sampling {target_bone_count} bones randomly from {len(potential_indices)} potential locations.")
        selected_indices = random.sample(potential_indices, target_bone_count)
    else:
        print(f"Using all {len(potential_indices)} potential locations (<= target {target_bone_count}).")
        selected_indices = potential_indices

    # Return the mesh vertex indices corresponding to the selected positions
    final_vertex_indices = selected_indices
    print(f"Final selected vertex indices for bone heads: {len(final_vertex_indices)}")
    return final_vertex_indices


def create_rig_from_vertices(mesh_obj, bone_head_indices, fixed_bone_length=0.005):
    """
    Creates an armature with bones at the specified vertex indices. # Translated docstring
    Uses normal orientation (like v1.5) and a smaller, fixed bone length.
    """
    if not bone_head_indices:
        print("No vertex indices provided for bone creation.")
        return None, {} # Return None and an empty dictionary

    mesh = mesh_obj.data
    world_matrix = mesh_obj.matrix_world

    # Check if mesh exists and has vertices
    if not mesh or not mesh.vertices:
        print(f"Error: Mesh '{mesh_obj.name}' seems invalid or has no vertices.")
        return None, {}

    # Ensure we are in object mode before adding
    if bpy.context.object and bpy.context.object.mode != 'OBJECT':
        bpy.ops.object.mode_set(mode='OBJECT')

    # Remember selection and active object
    original_active = bpy.context.view_layer.objects.active
    original_selection = bpy.context.selected_objects[:]

    bpy.ops.object.select_all(action='DESELECT')

    armature_obj = None
    armature_data = None
    created_bone_map = {} # Dictionary: Bone Name -> Original Vertex Index

    try:
        bpy.ops.object.add(type='ARMATURE', enter_editmode=True, align='WORLD', location=(0, 0, 0))
        armature_obj = bpy.context.object
        armature_obj.name = mesh_obj.name + "_Rig"
        armature_data = armature_obj.data
        armature_data.name = armature_obj.name + "_Data"

        # Ensure we are in armature edit mode
        if armature_obj.mode != 'EDIT':
            print("Warning: Could not enter edit mode automatically. Attempting manual switch.")
            bpy.ops.object.mode_set(mode='EDIT')

        # Remove default bones (if any)
        if armature_data.edit_bones:
            bone_names_to_remove = [b.name for b in armature_data.edit_bones]
            for name in bone_names_to_remove:
                bone_to_remove = armature_data.edit_bones.get(name)
                if bone_to_remove:
                    try: armature_data.edit_bones.remove(bone_to_remove)
                    except Exception as e: print(f"Warning: Could not remove default bone '{name}': {e}")

        created_bones_list = [] # List of created EditBones
        missing_verts = 0
        max_vert_index = len(mesh.vertices) - 1

        # Ensure normals are calculated (do it once)
        try:
            mesh.calc_normals_split() # Use split normals for potentially better vertex normal results
            # mesh.calc_normals() # Alternative standard method
            print("Mesh normals calculated.")
        except Exception as ne:
             print(f"Error calculating mesh normals: {ne}. Proceeding with potential issues.")

        for i, vert_index in enumerate(bone_head_indices):
            if vert_index > max_vert_index or vert_index < 0:
                print(f"Warning: Vertex index {vert_index} out of bounds (max is {max_vert_index}). Skipping.")
                missing_verts += 1
                continue

            vert = mesh.vertices[vert_index]
            head_pos = world_matrix @ vert.co

            # Calculate normal safely (similar to v1.5/v1.6)
            normal_world = Vector((0,0,1)) # Default fallback
            try:
                 normal_local = vert.normal
                 # Transform normal to world space
                 normal_world = (world_matrix.to_3x3().inverted_safe().transposed() @ normal_local).normalized()

                 if normal_world.length < 0.1: # Check for zero-length normal
                     print(f"Warning: Calculated normal for vertex {vert_index} is near zero. Using fallback.")
                     normal_world = Vector((0,0,1)) # Fallback normal
            except AttributeError: # Handles cases where .normal might not exist immediately after calc_normals_split sometimes?
                 print(f"Warning: Could not access normal for vertex {vert_index} directly. Using fallback.")
                 normal_world = Vector((0,0,1)) # Default upwards normal
            except Exception as e:
                print(f"Warning: Could not get or transform normal for vertex {vert_index}: {e}. Using fallback.")
                normal_world = Vector((0,0,1)) # Default upwards normal

            # Calculate tail position based on normal and FIXED SMALL length
            tail_pos = head_pos + normal_world * fixed_bone_length
            bone_name = f"Vtx_{vert_index}"

            # Avoid duplicate bone names robustly
            base_name = bone_name
            count = 1
            while bone_name in armature_data.edit_bones:
                 bone_name = f"{base_name}_{count}"
                 count += 1
                 if count > len(bone_head_indices) + 10: # Safety break
                      print(f"Error: Could not find unique name for bone based on Vtx_{vert_index}. Skipping.")
                      bone_name = None; break
            if bone_name is None:
                 missing_verts += 1; continue

            try:
                edit_bone = armature_data.edit_bones.new(bone_name)
                edit_bone.head = head_pos
                edit_bone.tail = tail_pos
                created_bone_map[bone_name] = vert_index
                created_bones_list.append(edit_bone) # Store the EditBone
            except Exception as e:
                print(f"Error creating bone '{bone_name}' for vertex {vert_index}: {e}")
                missing_verts += 1

        if missing_verts > 0: print(f"Skipped {missing_verts} bones (invalid vertex indices or errors).")

        if not created_bones_list:
            print("No bones were created.")
            if armature_obj.mode == 'EDIT':
               bpy.ops.object.mode_set(mode='OBJECT')
            # Clean up the created empty armature object
            bpy.ops.object.select_all(action='DESELECT')
            armature_obj.select_set(True)
            bpy.ops.object.delete(use_global=False) # Deletes selected (the armature)
            if armature_data and armature_data.users == 0:
                 bpy.data.armatures.remove(armature_data, do_unlink=True)
            # Restore selection
            bpy.ops.object.select_all(action='DESELECT')
            for obj in original_selection:
                if obj and obj.name in bpy.context.view_layer.objects: obj.select_set(True)
            if original_active and original_active.name in bpy.context.view_layer.objects:
                bpy.context.view_layer.objects.active = original_active
            return None, {}

        # Exit edit mode after creating all bones
        bpy.ops.object.mode_set(mode='OBJECT')
        print(f"Created {len(created_bones_list)} initial bones in armature '{armature_obj.name}'.")

        # Restore selection
        bpy.ops.object.select_all(action='DESELECT')
        for obj in original_selection:
            if obj and obj.name in bpy.context.view_layer.objects: obj.select_set(True)
        if original_active and original_active.name in bpy.context.view_layer.objects:
            bpy.context.view_layer.objects.active = original_active

        return armature_obj, created_bone_map

    except Exception as e:
        print(f"Critical error during armature creation: {e}")
        # Clean up if armature object was partially created
        if armature_obj and armature_obj.name in bpy.context.view_layer.objects:
            if bpy.context.object and bpy.context.object.mode != 'OBJECT': bpy.ops.object.mode_set(mode='OBJECT')
            bpy.ops.object.select_all(action='DESELECT')
            armature_obj.select_set(True)
            bpy.ops.object.delete(use_global=False)
            if armature_data and armature_data.users == 0: bpy.data.armatures.remove(armature_data, do_unlink=True)
        # Restore selection
        bpy.ops.object.select_all(action='DESELECT')
        for obj in original_selection:
            if obj and obj.name in bpy.context.view_layer.objects: obj.select_set(True)
        if original_active and original_active.name in bpy.context.view_layer.objects:
            bpy.context.view_layer.objects.active = original_active
        return None, {}


def parent_mesh_to_rig(mesh_obj, armature_obj):
    """Parents the mesh to the armature with automatic weights. (Unchanged from v1.6)""" # Translated docstring
    if not mesh_obj or not armature_obj:
        print("Mesh or Armature object not found for parenting.")
        return False

    if mesh_obj.name not in bpy.context.view_layer.objects or armature_obj.name not in bpy.context.view_layer.objects:
         print("Mesh or Armature object no longer in scene/view layer.")
         return False

    if not armature_obj.data.bones:
        print(f"Armature '{armature_obj.name}' has no bones. Skipping parenting.")
        return False

    if not mesh_obj.data.vertices:
        print(f"Mesh '{mesh_obj.name}' has no vertices. Skipping parenting.")
        return False

    original_active = bpy.context.view_layer.objects.active
    selected_objects = bpy.context.selected_objects[:]

    if bpy.context.object and bpy.context.object.mode != 'OBJECT':
        bpy.ops.object.mode_set(mode='OBJECT')

    bpy.ops.object.select_all(action='DESELECT')
    mesh_obj.select_set(True)
    armature_obj.select_set(True)
    bpy.context.view_layer.objects.active = armature_obj

    print(f"Parenting '{mesh_obj.name}' to '{armature_obj.name}' with automatic weights...")
    parenting_successful = False
    try:
        bpy.ops.object.parent_set(type='ARMATURE_AUTO')
        print("Parenting successful.")
        parenting_successful = True
    except RuntimeError as e:
        print(f"Error during automatic weighting: {e}")
        if mesh_obj.parent == armature_obj: mesh_obj.parent = None
        mod = mesh_obj.modifiers.get("Armature")
        if mod and mod.object == armature_obj: mesh_obj.modifiers.remove(mod)
        parenting_successful = False
    except Exception as e:
        print(f"Unexpected error during parenting: {e}")
        parenting_successful = False


    # Restore previous selection and active object
    bpy.ops.object.select_all(action='DESELECT')
    restored_active = False
    for obj in selected_objects:
        if obj and obj.name in bpy.context.view_layer.objects:
            try:
                 obj.select_set(True)
                 if obj == original_active and not restored_active:
                      bpy.context.view_layer.objects.active = obj
                      restored_active = True
            except ReferenceError: print(f"Warning: Could not re-select object '{obj.name if obj else 'N/A'}'.")

    if not restored_active and original_active and original_active.name in bpy.context.view_layer.objects:
         try: bpy.context.view_layer.objects.active = original_active
         except Exception as e: print(f"Warning: Could not restore original active object '{original_active.name}': {e}")
    elif not bpy.context.view_layer.objects.active:
         if armature_obj and armature_obj.name in bpy.context.view_layer.objects: bpy.context.view_layer.objects.active = armature_obj
         elif mesh_obj and mesh_obj.name in bpy.context.view_layer.objects: bpy.context.view_layer.objects.active = mesh_obj

    return parenting_successful


# --- Rig Creation Operator (Uses updated create_rig_from_vertices) ---
class RIG_OT_CreateMeshRig(bpy.types.Operator):
    """Creates an armature for the selected mesh based on vertex proximity.""" # Translated docstring
    bl_idname = "object.create_mesh_rig"
    bl_label = "Create Rig" # Translated label
    bl_options = {'REGISTER', 'UNDO'}

    bone_length: bpy.props.FloatProperty(
        name="Bone Length", # Translated name
        description="Fixed length for the created bones", # Translated description
        default=0.005, min=0.0001, max=1.0, subtype='DISTANCE', unit='LENGTH', precision=4
    )

    @classmethod
    def poll(cls, context):
        return get_selected_mesh_object() is not None

    def execute(self, context):
        start_time = time.time()
        mesh_obj = get_selected_mesh_object()
        if not mesh_obj:
            self.report({'ERROR'}, "No MESH object found.") # Translated report
            return {'CANCELLED'}

        scene = context.scene
        target_bone_count = scene.rig_tool_bone_count
        proximity_threshold = scene.rig_tool_proximity_threshold
        fixed_bone_length = self.bone_length # Use the operator property

        # --- Checks ---
        if target_bone_count <= 0: self.report({'ERROR'}, "Bone Count must be > 0."); return {'CANCELLED'} # Translated report
        if proximity_threshold < 0.0: self.report({'ERROR'}, "Proximity Threshold must be >= 0."); return {'CANCELLED'} # Translated report
        if fixed_bone_length <= 0: self.report({'ERROR'}, "Bone Length must be > 0."); return {'CANCELLED'} # Translated report

        self.report({'INFO'}, f"Starting rig creation for '{mesh_obj.name}'...") # Translated report
        self.report({'INFO'}, f"Parameters: Bones={target_bone_count}, Threshold={proximity_threshold:.4f}, Length={fixed_bone_length:.4f}") # Translated report

        # 1. Select vertices
        selected_indices = select_bone_vertices(mesh_obj, target_bone_count, proximity_threshold)
        if not selected_indices:
            self.report({'WARNING'}, "Could not select vertices for bones.") # Translated report
            return {'CANCELLED'}

        # 2. Create armature (using the updated logic with fixed length)
        armature_obj, bone_vertex_map = create_rig_from_vertices(mesh_obj, selected_indices, fixed_bone_length=fixed_bone_length)
        if not armature_obj:
            self.report({'ERROR'}, "Failed to create armature.") # Translated report
            if mesh_obj.name in context.view_layer.objects:
                 if context.view_layer.objects.active != mesh_obj:
                     try: context.view_layer.objects.active = mesh_obj
                     except: pass
            return {'CANCELLED'}

        final_bone_count = len(armature_obj.data.bones) if armature_obj.data else 0
        end_time = time.time()
        report_msg = f"Rig '{armature_obj.name}' created ({final_bone_count} bones). Time: {end_time - start_time:.2f}s. Mesh NOT parented." # Translated report
        self.report({'INFO'}, report_msg)

        # --- Set targets for the next steps ---
        scene.rig_tool_parenting_mesh_target = mesh_obj
        scene.rig_tool_parenting_armature_target = armature_obj
        scene.rig_tool_constraint_target = mesh_obj # Default constraint target to original mesh
        scene.rig_tool_surfdeform_source_mesh = mesh_obj # Default SurfDeform source to original mesh
        scene.rig_tool_surfdeform_target_mesh = mesh_obj # Target for SurfDeform often same as source initially
        scene.rig_tool_bake_armature_target = armature_obj # Default Bake target to new armature
        scene.rig_tool_final_source_mesh = mesh_obj # Default Finalize source to original mesh
        scene.rig_tool_final_target_armature = armature_obj # Default Finalize target to new armature

        # Select the created armature
        bpy.ops.object.select_all(action='DESELECT')
        if armature_obj.name in context.view_layer.objects:
            armature_obj.select_set(True)
            context.view_layer.objects.active = armature_obj
        else:
             self.report({'WARNING'}, f"Created armature '{armature_obj.name}' not found after creation.") # Translated report

        return {'FINISHED'}

# --- Operator to merge SELECTED bones in Edit Mode (Unchanged from v1.6) ---
class RIG_OT_MergeSelectedBones(bpy.types.Operator):
    """Merges selected bones in Edit Mode if they are close enough""" # Translated docstring
    bl_idname = "armature.merge_selected_bones_tool"
    bl_label = "Merge Selected Bones" # Translated label
    bl_options = {'REGISTER', 'UNDO'}

    @classmethod
    def poll(cls, context):
        if not (context.object and context.object.type == 'ARMATURE' and context.mode == 'EDIT_ARMATURE'): return False
        scene = context.scene
        min_count = scene.rig_tool_merge_min_count
        return context.selected_editable_bones and len(context.selected_editable_bones) >= min_count

    def execute(self, context):
        start_time = time.time()
        armature_obj = context.object
        armature_data = armature_obj.data
        scene = context.scene

        proximity_threshold = scene.rig_tool_proximity_threshold
        merge_factor = scene.rig_tool_merge_threshold_factor
        min_merge_count = scene.rig_tool_merge_min_count

        if merge_factor <= 1.0: self.report({'WARNING'}, "Merge Factor must be > 1.0."); return {'CANCELLED'} # Translated report
        if min_merge_count < 2: self.report({'ERROR'}, "Min Bones to Merge must be >= 2."); return {'CANCELLED'} # Translated report

        merge_radius = proximity_threshold * merge_factor
        self.report({'INFO'}, f"Starting merge of selected bones. Radius: {merge_radius:.4f}, Min Count: {min_merge_count}") # Translated report

        selected_bones = [b for b in armature_data.edit_bones if b.select]
        if len(selected_bones) < min_merge_count:
            self.report({'INFO'}, f"Not enough bones selected ({len(selected_bones)}/{min_merge_count})."); return {'CANCELLED'} # Translated report

        bone_data = {}
        bone_heads = []
        bone_names_ordered = []

        for bone in selected_bones:
            if bone.name not in armature_data.edit_bones: continue
            head = bone.head.copy()
            tail = bone.tail.copy()
            length = bone.length
            direction = bone.vector.normalized() if length > 1e-6 else Vector((0,0,1))
            bone_data[bone.name] = {'head': head, 'tail': tail, 'length': length, 'dir': direction, 'bone': bone}
            bone_heads.append(head)
            bone_names_ordered.append(bone.name)

        if not bone_heads: self.report({'WARNING'}, "Could not find heads of selected bones."); return {'CANCELLED'} # Translated report

        kdt = KDTree(len(bone_heads))
        for i, head_pos in enumerate(bone_heads): kdt.insert(head_pos, i)
        kdt.balance()

        processed_bone_indices = set()
        bones_to_remove_names = set()
        bones_to_add_data = []
        merged_bone_names = set()
        merge_counter = 0

        for i, bone_name in enumerate(bone_names_ordered):
            if i in processed_bone_indices or bone_name not in bone_data: continue

            current_head = bone_heads[i]
            cluster_results = kdt.find_range(current_head, merge_radius)
            actual_cluster_indices = {idx for pos, idx, dist_sq in cluster_results if idx not in processed_bone_indices and bone_names_ordered[idx] in bone_data}

            if len(actual_cluster_indices) >= min_merge_count:
                merge_counter += 1
                actual_cluster_names = [bone_names_ordered[idx] for idx in actual_cluster_indices]
                self.report({'INFO'}, f"  Cluster {merge_counter} ({len(actual_cluster_names)}): {', '.join(actual_cluster_names)}") # Translated report

                avg_head, avg_dir = Vector((0,0,0)), Vector((0,0,0))
                total_length, num_in_cluster = 0.0, len(actual_cluster_indices)
                cluster_edit_bone_names = set()

                for idx in actual_cluster_indices:
                    name = bone_names_ordered[idx]; data = bone_data[name]
                    avg_head += data['head']; avg_dir += data['dir']; total_length += data['length']
                    cluster_edit_bone_names.add(name)

                avg_head /= num_in_cluster
                avg_length = total_length / num_in_cluster
                if avg_dir.length > 1e-6: avg_dir.normalize()
                else: avg_dir = Vector((0,0,1))
                final_bone_length = max(avg_length, 0.001)
                avg_tail = avg_head + avg_dir * final_bone_length

                new_bone_name_base = f"Merged_{merge_counter:03d}"; new_bone_name = new_bone_name_base; k = 1
                existing_bone_names = {b.name for b in armature_data.edit_bones} | merged_bone_names
                while new_bone_name in existing_bone_names: new_bone_name = f"{new_bone_name_base}_{k}"; k += 1

                bones_to_add_data.append({'name': new_bone_name, 'head': avg_head, 'tail': avg_tail})
                merged_bone_names.add(new_bone_name)
                bones_to_remove_names.update(cluster_edit_bone_names)
                processed_bone_indices.update(actual_cluster_indices)
                for name_to_remove in cluster_edit_bone_names:
                     if name_to_remove in bone_data: del bone_data[name_to_remove]
                self.report({'INFO'}, f"    -> Merging into '{new_bone_name}' at {avg_head.to_tuple(3)}") # Translated report
            else:
                processed_bone_indices.add(i)

        removed_count = 0
        if bones_to_remove_names:
            self.report({'INFO'}, f"Removing {len(bones_to_remove_names)} bones...") # Translated report
            for name in list(bones_to_remove_names):
                 bone_to_remove = armature_data.edit_bones.get(name)
                 if bone_to_remove:
                     try: armature_data.edit_bones.remove(bone_to_remove); removed_count += 1
                     except Exception as e: self.report({'WARNING'}, f"  Error removing bone '{name}': {e}") # Translated report
            self.report({'INFO'}, f"Removed {removed_count} bones.") # Translated report

        added_count = 0; newly_added_bone_names = []
        if bones_to_add_data:
            self.report({'INFO'}, f"Adding {len(bones_to_add_data)} merged bones...") # Translated report
            for data in bones_to_add_data:
                 final_name = data['name']; k=1
                 while final_name in armature_data.edit_bones: final_name = f"{data['name']}_dup{k}"; k+=1
                 if final_name != data['name']: self.report({'INFO'}, f"  Name changed to '{final_name}'.") # Translated report
                 try:
                     new_bone = armature_data.edit_bones.new(final_name)
                     new_bone.head, new_bone.tail = data['head'], data['tail']
                     newly_added_bone_names.append(final_name); added_count += 1
                 except Exception as e: self.report({'ERROR'}, f"  Error adding bone '{final_name}': {e}") # Translated report
            self.report({'INFO'}, f"Added {added_count} merged bones.") # Translated report

        bpy.ops.armature.select_all(action='DESELECT')
        for name in newly_added_bone_names:
            new_bone = armature_data.edit_bones.get(name)
            if new_bone: new_bone.select, new_bone.select_head, new_bone.select_tail = True, True, True

        context.view_layer.objects.active = armature_obj; armature_obj.select_set(True)
        end_time = time.time()
        self.report({'INFO'}, f"Merge complete. Removed: {removed_count}, Added: {added_count}. Time: {end_time - start_time:.2f}s.") # Translated report
        return {'FINISHED'}

# --- Mesh Parenting Operator (Unchanged from v1.6) ---
class RIG_OT_ParentMeshToRigTool(bpy.types.Operator):
    """Parents the selected mesh to the selected armature with automatic weights""" # Translated docstring
    bl_idname = "object.parent_mesh_to_rig_tool"
    bl_label = "Parent Mesh to Armature" # Translated label
    bl_options = {'REGISTER', 'UNDO'}

    @classmethod
    def poll(cls, context):
        scene = context.scene
        mesh_target = scene.rig_tool_parenting_mesh_target
        armature_target = scene.rig_tool_parenting_armature_target
        return (mesh_target and armature_target and
                mesh_target.name in context.view_layer.objects and
                armature_target.name in context.view_layer.objects and
                mesh_target.type == 'MESH' and armature_target.type == 'ARMATURE')

    def execute(self, context):
        scene = context.scene
        mesh_obj = scene.rig_tool_parenting_mesh_target
        armature_obj = scene.rig_tool_parenting_armature_target

        if not mesh_obj or mesh_obj.name not in context.view_layer.objects: self.report({'ERROR'}, "Target Mesh not set/found."); return {'CANCELLED'} # Translated report
        if not armature_obj or armature_obj.name not in context.view_layer.objects: self.report({'ERROR'}, "Target Armature not set/found."); return {'CANCELLED'} # Translated report
        if mesh_obj.type != 'MESH': self.report({'ERROR'}, f"'{mesh_obj.name}' is not a Mesh."); return {'CANCELLED'} # Translated report
        if armature_obj.type != 'ARMATURE': self.report({'ERROR'}, f"'{armature_obj.name}' is not an Armature."); return {'CANCELLED'} # Translated report

        self.report({'INFO'}, f"Parenting '{mesh_obj.name}' to '{armature_obj.name}'...") # Translated report
        parenting_successful = parent_mesh_to_rig(mesh_obj, armature_obj)

        if parenting_successful:
            self.report({'INFO'}, "Mesh parenting to armature complete.") # Translated report
            # Update SurfDeform target if it was still pointing to the source
            if scene.rig_tool_surfdeform_target_mesh == scene.rig_tool_surfdeform_source_mesh:
                scene.rig_tool_surfdeform_target_mesh = mesh_obj
                print(f"Updated SurfDeform Target to '{mesh_obj.name}' after parenting.")
        else:
            self.report({'ERROR'}, "Error during mesh parenting.") # Translated report
            return {'CANCELLED'}

        return {'FINISHED'}


# --- Operator: Constraints -> Apply Pose -> Remove Mod (Unchanged from v1.6) ---
class RIG_OT_AddBoneConstraints(bpy.types.Operator):
    """Adds constraints, applies current pose as Rest Pose, removes Armature modifier from mesh""" # Translated docstring
    bl_idname = "object.add_bone_constraints"
    bl_label = "Constr -> Apply Pose -> Del Mod" # Translated label (kept short)
    bl_options = {'REGISTER', 'UNDO'}

    constraint_type: bpy.props.StringProperty(default='COPY_LOCATION') # Can be changed if needed

    @classmethod
    def poll(cls, context):
        armature_obj = get_selected_armature_object()
        target_mesh_vg = context.scene.rig_tool_constraint_target
        return (armature_obj and armature_obj.pose and
                target_mesh_vg and target_mesh_vg.type == 'MESH' and
                target_mesh_vg.name in context.view_layer.objects)

    def execute(self, context):
        start_time = time.time()
        armature_obj = get_selected_armature_object()
        target_mesh_vg = context.scene.rig_tool_constraint_target
        mesh_parented = context.scene.rig_tool_parenting_mesh_target # Mesh to remove modifier from
        scene = context.scene

        if not armature_obj: self.report({'ERROR'}, "No ARMATURE selected."); return {'CANCELLED'} # Translated report
        if not target_mesh_vg: self.report({'ERROR'}, "Target Mesh (VG) not selected."); return {'CANCELLED'} # Translated report
        if target_mesh_vg.name not in context.view_layer.objects: self.report({'ERROR'}, f"Target Mesh (VG) '{target_mesh_vg.name}' not found."); return {'CANCELLED'} # Translated report
        if target_mesh_vg.type != 'MESH': self.report({'ERROR'}, "Target (VG) must be a MESH."); return {'CANCELLED'} # Translated report
        if not armature_obj.pose: self.report({'ERROR'}, f"Armature '{armature_obj.name}' has no Pose."); return {'CANCELLED'} # Translated report
        if not target_mesh_vg.vertex_groups: self.report({'WARNING'}, f"Mesh '{target_mesh_vg.name}' (VG Target) has no vertex groups.") # Translated report
        if not armature_obj.data.bones: self.report({'WARNING'}, f"Armature '{armature_obj.name}' has no bones."); return {'CANCELLED'} # Translated report

        self.report({'INFO'}, f"Step 3: Constraints -> Pose -> Remove Mod...") # Translated report
        self.report({'INFO'}, f"  Armature: '{armature_obj.name}', VG Target: '{target_mesh_vg.name}', Mod Mesh: '{mesh_parented.name if mesh_parented else 'None'}'") # Translated report

        original_mode = armature_obj.mode
        original_active = context.view_layer.objects.active
        original_selection = context.selected_objects[:]
        constraint_count, missing_vg_count, apply_pose_success, removed_modifier_flag = 0, 0, False, False

        try:
            bpy.ops.object.select_all(action='DESELECT')
            context.view_layer.objects.active = armature_obj; armature_obj.select_set(True)
            if original_mode != 'POSE': bpy.ops.object.mode_set(mode='POSE')

            # --- 1. Add/Update Constraints ---
            self.report({'INFO'}, "Adding/updating constraints...") # Translated report
            for pbone in armature_obj.pose.bones:
                bone_name = pbone.name; target_vg_name = bone_name
                vertex_group = target_mesh_vg.vertex_groups.get(target_vg_name)
                if vertex_group:
                    existing_constraint = next((c for c in pbone.constraints if c.type == self.constraint_type and c.name.startswith("RigTool_")), None)
                    if existing_constraint:
                        try: pbone.constraints.remove(existing_constraint);
                        except Exception as rem_e: print(f"Warn: Could not remove existing constraint on '{bone_name}': {rem_e}") # Kept print message
                    try:
                        constraint = pbone.constraints.new(type=self.constraint_type); constraint.name = f"RigTool_{self.constraint_type}"
                        constraint.target = target_mesh_vg; constraint.subtarget = vertex_group.name
                        if self.constraint_type == 'COPY_LOCATION': constraint.use_x, constraint.use_y, constraint.use_z = True, True, True
                        elif self.constraint_type == 'COPY_ROTATION': constraint.use_x, constraint.use_y, constraint.use_z = True, True, True # Example
                        constraint_count += 1
                    except Exception as e: print(f"Error adding constraint to bone '{bone_name}': {e}") # Kept print message
                else: missing_vg_count += 1 # print(f"Info: No VG '{target_vg_name}' on '{target_mesh_vg.name}' for bone '{bone_name}'.")
            self.report({'INFO'}, f"Added/updated {constraint_count} constraints (skipped {missing_vg_count} missing VGs).") # Translated report

            # --- 2. Apply Pose as Rest Pose ---
            self.report({'INFO'}, f"Applying pose as Rest Pose for '{armature_obj.name}'...") # Translated report
            try:
                bpy.ops.pose.select_all(action='SELECT'); bpy.ops.pose.armature_apply(selected=False)
                self.report({'INFO'}, "Apply Rest Pose successful."); apply_pose_success = True # Translated report
            except RuntimeError as e: self.report({'ERROR'}, f"Error applying Rest Pose: {e}"); raise # Translated report
            except Exception as e: self.report({'ERROR'}, f"Unexpected error during Rest Pose: {e}"); raise # Translated report

            # --- 3. Remove Armature Modifier (AFTER Apply Pose) ---
            if mesh_parented and mesh_parented.name in context.view_layer.objects and mesh_parented.type == 'MESH':
                self.report({'INFO'}, f"Removing Armature mod from '{mesh_parented.name}' (linked to '{armature_obj.name}')...") # Translated report
                modifier_to_remove = None
                original_parented_mesh_mode = mesh_parented.mode
                if original_parented_mesh_mode != 'OBJECT':
                     active_before_switch = context.view_layer.objects.active
                     context.view_layer.objects.active = mesh_parented
                     bpy.ops.object.mode_set(mode='OBJECT')
                     context.view_layer.objects.active = active_before_switch # Restore active

                for mod in mesh_parented.modifiers:
                    if mod.type == 'ARMATURE' and mod.object == armature_obj: modifier_to_remove = mod; break
                if modifier_to_remove:
                    try: mod_name = modifier_to_remove.name; mesh_parented.modifiers.remove(modifier_to_remove); self.report({'INFO'}, f"Removed Armature mod ('{mod_name}') from '{mesh_parented.name}'."); removed_modifier_flag = True # Translated report
                    except Exception as mod_rem_e: self.report({'WARNING'}, f"Could not remove Armature mod ('{modifier_to_remove.name}') from '{mesh_parented.name}': {mod_rem_e}") # Translated report
                else: self.report({'INFO'}, f"No Armature mod for '{armature_obj.name}' found on '{mesh_parented.name}'.") # Translated report
                if mesh_parented.mode != original_parented_mesh_mode: # Restore mode if changed
                    active_before_switch = context.view_layer.objects.active
                    context.view_layer.objects.active = mesh_parented
                    bpy.ops.object.mode_set(mode=original_parented_mesh_mode)
                    context.view_layer.objects.active = active_before_switch # Restore active
                if context.view_layer.objects.active != armature_obj: # Ensure armature is active again
                     context.view_layer.objects.active = armature_obj; armature_obj.select_set(True)
            elif mesh_parented: self.report({'WARNING'}, f"Mesh '{mesh_parented.name}' for modifier removal not found or not a Mesh.") # Translated report
            else: self.report({'INFO'}, "Mesh for parenting/modifier removal not specified.") # Translated report

        except Exception as e:
            self.report({'ERROR'}, f"Error during Step 3: {e}") # Translated report
            try: # Attempt restoration on error
                if armature_obj and armature_obj.name in context.view_layer.objects and armature_obj.mode != original_mode: bpy.ops.object.mode_set(mode=original_mode)
                if context.object and context.object.mode != 'OBJECT': bpy.ops.object.mode_set(mode='OBJECT')
                bpy.ops.object.select_all(action='DESELECT')
                for obj in original_selection:
                     if obj and obj.name in context.view_layer.objects: obj.select_set(True)
                if original_active and original_active.name in context.view_layer.objects: context.view_layer.objects.active = original_active
                elif armature_obj and armature_obj.name in context.view_layer.objects: context.view_layer.objects.active = armature_obj
            except Exception as sel_e: print(f"Error restoring state after main error: {sel_e}") # Kept print message
            return {'CANCELLED'}
        finally: # Restore Original State (always attempt)
             try:
                 if armature_obj and armature_obj.name in context.view_layer.objects and armature_obj.mode != original_mode: bpy.ops.object.mode_set(mode=original_mode)
                 if context.object and context.object.mode != 'OBJECT': bpy.ops.object.mode_set(mode='OBJECT')
                 bpy.ops.object.select_all(action='DESELECT'); restored_active_flag = False
                 for obj in original_selection:
                     if obj and obj.name in context.view_layer.objects:
                         obj.select_set(True)
                         if obj == original_active: context.view_layer.objects.active = obj; restored_active_flag = True
                 if not restored_active_flag and original_active and original_active.name in context.view_layer.objects: context.view_layer.objects.active = original_active
                 elif not context.view_layer.objects.active:
                      if armature_obj and armature_obj.name in context.view_layer.objects: context.view_layer.objects.active = armature_obj
             except Exception as e: print(f"Warning: Error during final state restoration: {e}") # Kept print message

        end_time = time.time()
        report_msg = f"Step 3 Done. C:{constraint_count}/{missing_vg_count}(add/skip). P:{'Ok' if apply_pose_success else 'Err'}. M:{'Ok' if removed_modifier_flag else 'No'}. T:{end_time-start_time:.1f}s." # Translated report (concise status)
        if not apply_pose_success or (mesh_parented and not removed_modifier_flag): self.report({'WARNING'}, report_msg)
        else: self.report({'INFO'}, report_msg)
        return {'FINISHED'} if apply_pose_success else {'CANCELLED'}

# --- Surface Deform Add Operator (Unchanged from v1.6) ---
class RIG_OT_AddSurfaceDeform(bpy.types.Operator):
    """Adds a SurfaceDeform modifier and performs Bind""" # Translated docstring
    bl_idname = "object.add_surface_deform_tool"
    bl_label = "Add SurfaceDeform & Bind" # Translated label
    bl_options = {'REGISTER', 'UNDO'}

    modifier_name: bpy.props.StringProperty(default="RigMeshPlus_SurfaceDeform")

    @classmethod
    def poll(cls, context):
        scene = context.scene
        source_mesh = scene.rig_tool_surfdeform_source_mesh
        target_mesh = scene.rig_tool_surfdeform_target_mesh
        return (source_mesh and source_mesh.name in context.view_layer.objects and source_mesh.type == 'MESH' and
                target_mesh and target_mesh.name in context.view_layer.objects and target_mesh.type == 'MESH')

    def execute(self, context):
        scene = context.scene
        source_mesh = scene.rig_tool_surfdeform_source_mesh
        target_mesh = scene.rig_tool_surfdeform_target_mesh

        if not source_mesh or source_mesh.name not in context.view_layer.objects or source_mesh.type != 'MESH': self.report({'ERROR'}, "Source mesh not found or not a mesh."); return {'CANCELLED'} # Translated report
        if not target_mesh or target_mesh.name not in context.view_layer.objects or target_mesh.type != 'MESH': self.report({'ERROR'}, "Target mesh not found or not a mesh."); return {'CANCELLED'} # Translated report
        if source_mesh == target_mesh: self.report({'WARNING'}, "Source and Target meshes are the same.") # Translated report

        self.report({'INFO'}, f"Adding SurfaceDeform to '{source_mesh.name}', target '{target_mesh.name}'...") # Translated report

        original_active = context.view_layer.objects.active
        original_selection = context.selected_objects[:]
        original_mode = None
        if source_mesh.name in context.view_layer.objects: original_mode = source_mesh.mode
        mod, bind_success = None, False

        try:
             if source_mesh.mode != 'OBJECT': bpy.ops.object.mode_set(mode='OBJECT')
             mod = source_mesh.modifiers.get(self.modifier_name)
             if mod:
                 self.report({'INFO'}, f"Modifier '{self.modifier_name}' exists. Updating target.") # Translated report
                 if mod.type != 'SURFACE_DEFORM': self.report({'ERROR'}, f"'{self.modifier_name}' is not a SurfaceDeform modifier."); return {'CANCELLED'} # Translated report
                 if mod.is_bound and mod.target != target_mesh:
                     self.report({'INFO'}, "Unbinding due to target change.") # Translated report
                     bpy.ops.object.select_all(action='DESELECT')
                     context.view_layer.objects.active = source_mesh; source_mesh.select_set(True)
                     try: bpy.ops.object.surfacedeform_bind(modifier=mod.name) # Toggles
                     except RuntimeError as unbind_e: print(f"Info: Could not unbind cleanly: {unbind_e}") # Kept print message
             else:
                 mod = source_mesh.modifiers.new(name=self.modifier_name, type='SURFACE_DEFORM')
                 self.report({'INFO'}, f"Modifier '{self.modifier_name}' added.") # Translated report
             mod.target = target_mesh

             self.report({'INFO'}, f"Applying Bind for '{mod.name}'...") # Translated report
             bpy.ops.object.select_all(action='DESELECT')
             context.view_layer.objects.active = source_mesh; source_mesh.select_set(True)
             if mod.is_bound and mod.target == target_mesh:
                 self.report({'INFO'}, "Modifier is already bound to the target."); bind_success = True # Translated report
             else:
                 bpy.ops.object.surfacedeform_bind(modifier=mod.name)
                 context.view_layer.update()
                 mod = source_mesh.modifiers.get(self.modifier_name) # Re-get status
                 if mod and mod.is_bound: self.report({'INFO'}, "Bind applied successfully."); bind_success = True # Translated report
                 else: self.report({'ERROR'}, f"Failed to apply Bind for '{mod.name if mod else 'N/A'}'. Check intersection/topology.") # Translated report

        except RuntimeError as e: self.report({'ERROR'}, f"SurfaceDeform/Bind error: {e}"); bind_success = False # Translated report
        except Exception as e: self.report({'ERROR'}, f"Unexpected error: {e}"); bind_success = False # Translated report
        finally: # Restore State
            try:
                if original_mode is not None and source_mesh.name in context.view_layer.objects and source_mesh.mode != original_mode: bpy.ops.object.mode_set(mode=original_mode)
                if context.object and context.object.mode != 'OBJECT': bpy.ops.object.mode_set(mode='OBJECT')
                bpy.ops.object.select_all(action='DESELECT'); restored_active_flag = False
                for obj in original_selection:
                     if obj and obj.name in context.view_layer.objects:
                         obj.select_set(True)
                         if obj == original_active: context.view_layer.objects.active = obj; restored_active_flag = True
                if not restored_active_flag and original_active and original_active.name in context.view_layer.objects: context.view_layer.objects.active = original_active
                elif not context.view_layer.objects.active:
                     if source_mesh and source_mesh.name in context.view_layer.objects: context.view_layer.objects.active = source_mesh
                     elif original_active and original_active.name in context.view_layer.objects: context.view_layer.objects.active = original_active
            except Exception as e: print(f"Warning: Error during SD state restoration: {e}") # Kept print message

        return {'FINISHED'} if bind_success else {'CANCELLED'}

# --- Animation Baking Operator (Updated for v1.7.1 parameters) ---
class RIG_OT_BakeRigAction(bpy.types.Operator):
    """Bakes animation for the selected armature with specified parameters""" # Translated docstring
    bl_idname = "object.bake_rig_action_tool"
    bl_label = "Bake Rig Action" # Translated label
    bl_options = {'REGISTER', 'UNDO'}

    # --- Operator properties - now fixed as requested ---
    # visual_keying: bpy.props.BoolProperty(default=True) # Now always True
    # clear_constraints: bpy.props.BoolProperty(default=True) # Now always True
    # use_current_action: bpy.props.BoolProperty(default=True) # Added and always True
    # bake_types: bpy.props.StringProperty(default='POSE') # Now always POSE
    # overwrite_current_action removed from nla.bake

    @classmethod
    def poll(cls, context):
        scene = context.scene
        armature_target = scene.rig_tool_bake_armature_target
        return (armature_target and armature_target.name in context.view_layer.objects and
                armature_target.type == 'ARMATURE' and armature_target.pose)

    def execute(self, context):
        scene = context.scene
        armature_obj = scene.rig_tool_bake_armature_target
        start_frame = scene.rig_tool_bake_start_frame # Still taken from UI
        end_frame = scene.rig_tool_bake_end_frame   # Still taken from UI

        if not armature_obj or armature_obj.name not in context.view_layer.objects or armature_obj.type != 'ARMATURE': self.report({'ERROR'}, "Target armature not found."); return {'CANCELLED'} # Translated report
        if not armature_obj.pose or not armature_obj.pose.bones: self.report({'ERROR'}, f"Armature '{armature_obj.name}' has no Pose Bones."); return {'CANCELLED'} # Translated report
        if end_frame < start_frame: self.report({'ERROR'}, "End frame is before start frame."); return {'CANCELLED'} # Translated report

        # --- Use fixed parameters as requested ---
        fixed_visual_keying = True
        fixed_clear_constraints = True
        fixed_use_current_action = True
        fixed_bake_types = {'POSE'}

        self.report({'INFO'}, f"Baking animation for '{armature_obj.name}' from {start_frame} to {end_frame}...") # Translated report
        # Updated INFO message to reflect fixed options
        self.report({'INFO'}, f"  Options: VisualKeying={fixed_visual_keying}, ClearCons={fixed_clear_constraints}, UseCurrentAction={fixed_use_current_action}, BakeTypes={fixed_bake_types}") # Translated report

        original_active = context.view_layer.objects.active
        original_selection = context.selected_objects[:]
        original_mode = armature_obj.mode if armature_obj.name in context.view_layer.objects else None
        original_frame = scene.frame_current

        try:
            bpy.ops.object.select_all(action='DESELECT')
            context.view_layer.objects.active = armature_obj; armature_obj.select_set(True)
            if armature_obj.mode != 'POSE': bpy.ops.object.mode_set(mode='POSE')

            # --- CALL BAKE WITH SPECIFIED PARAMETERS ---
            bpy.ops.nla.bake(
                frame_start=start_frame,              # Taken from UI
                frame_end=end_frame,                # Taken from UI
                step=1,                               # Standard step
                only_selected=True,                  # As requested (and previously)
                visual_keying=fixed_visual_keying,     # As requested (True)
                clear_constraints=fixed_clear_constraints, # As requested (True)
                clear_parents=False,                 # Standard False
                use_current_action=fixed_use_current_action, # As requested (True)
                # overwrite_current_action=...       # This parameter is removed from the call
                bake_types=fixed_bake_types          # As requested ({'POSE'})
            )
            self.report({'INFO'}, "Baking completed successfully.") # Translated report

        except RuntimeError as e: self.report({'ERROR'}, f"Error during baking: {e}"); return {'CANCELLED'} # Translated report
        except Exception as e: self.report({'ERROR'}, f"Unexpected error during baking: {e}"); return {'CANCELLED'} # Translated report
        finally: # Restore State (no changes needed here)
             try:
                  scene.frame_set(original_frame)
                  if original_mode is not None and armature_obj.name in context.view_layer.objects and armature_obj.mode != original_mode: bpy.ops.object.mode_set(mode=original_mode)
                  if context.object and context.object.mode != 'OBJECT': bpy.ops.object.mode_set(mode='OBJECT')
                  bpy.ops.object.select_all(action='DESELECT'); restored_active_flag = False
                  for obj in original_selection:
                       if obj and obj.name in context.view_layer.objects:
                           obj.select_set(True)
                           if obj == original_active: context.view_layer.objects.active = obj; restored_active_flag = True
                  if not restored_active_flag and original_active and original_active.name in context.view_layer.objects: context.view_layer.objects.active = original_active
                  elif not context.view_layer.objects.active:
                        if armature_obj and armature_obj.name in context.view_layer.objects: context.view_layer.objects.active = armature_obj
                        elif original_active and original_active.name in context.view_layer.objects: context.view_layer.objects.active = original_active
             except Exception as e: print(f"Warning: Error during Bake state restoration: {e}") # Kept print message
        return {'FINISHED'}


# --- NEW Step 6 Operator: Finalize Deformation ---
class RIG_OT_FinalizeDeform(bpy.types.Operator):
    """Removes SurfaceDeform and adds Armature modifier to the Source Mesh""" # Translated docstring
    bl_idname = "object.finalize_deform_tool"
    bl_label = "Remove SD, Add Armature Mod" # Translated label
    bl_options = {'REGISTER', 'UNDO'}

    surfdeform_modifier_name: bpy.props.StringProperty(
        name="SurfaceDeform Name to Remove", # Translated name
        default="RigMeshPlus_SurfaceDeform"
    )
    armature_modifier_name: bpy.props.StringProperty(
        name="New Armature Modifier Name", # Translated name
        default="Final_Armature_Deform"
    )

    @classmethod
    def poll(cls, context):
        scene = context.scene
        source_mesh = scene.rig_tool_final_source_mesh
        target_armature = scene.rig_tool_final_target_armature
        return (source_mesh and source_mesh.name in context.view_layer.objects and source_mesh.type == 'MESH' and
                target_armature and target_armature.name in context.view_layer.objects and target_armature.type == 'ARMATURE')

    def execute(self, context):
        scene = context.scene
        source_mesh = scene.rig_tool_final_source_mesh
        target_armature = scene.rig_tool_final_target_armature

        # --- Validation ---
        if not source_mesh or source_mesh.name not in context.view_layer.objects or source_mesh.type != 'MESH': self.report({'ERROR'}, "Source mesh not found or not a mesh."); return {'CANCELLED'} # Translated report
        if not target_armature or target_armature.name not in context.view_layer.objects or target_armature.type != 'ARMATURE': self.report({'ERROR'}, "Target armature not found or not an armature."); return {'CANCELLED'} # Translated report

        self.report({'INFO'}, f"Step 6: Finalizing for '{source_mesh.name}' with armature '{target_armature.name}'...") # Translated report

        # --- Prepare ---
        original_active = context.view_layer.objects.active
        original_selection = context.selected_objects[:]
        original_mode = source_mesh.mode if source_mesh.name in context.view_layer.objects else None
        removed_sd = False
        added_armature = False

        try:
            # Ensure Object mode for modifier operations
            if source_mesh.mode != 'OBJECT':
                active_before = context.view_layer.objects.active
                context.view_layer.objects.active = source_mesh
                bpy.ops.object.mode_set(mode='OBJECT')
                context.view_layer.objects.active = active_before # Restore active object


            # --- 1. Remove Surface Deform Modifier ---
            sd_mod = source_mesh.modifiers.get(self.surfdeform_modifier_name)
            if sd_mod and sd_mod.type == 'SURFACE_DEFORM':
                self.report({'INFO'}, f"Removing SurfaceDeform modifier '{sd_mod.name}'...") # Translated report
                try:
                    source_mesh.modifiers.remove(sd_mod)
                    removed_sd = True
                    self.report({'INFO'}, "SurfaceDeform removed.") # Translated report
                except Exception as e:
                    self.report({'WARNING'}, f"Could not remove SurfaceDeform '{sd_mod.name}': {e}") # Translated report
            elif sd_mod:
                 self.report({'WARNING'}, f"Found modifier '{self.surfdeform_modifier_name}', but it's not type SurfaceDeform.") # Translated report
            else:
                self.report({'INFO'}, f"SurfaceDeform modifier '{self.surfdeform_modifier_name}' not found on '{source_mesh.name}'.") # Translated report

            # --- 2. Remove potentially existing Armature mod pointing to the SAME target ---
            # Avoids adding duplicate modifiers if run multiple times
            existing_arm_mod = None
            for mod in source_mesh.modifiers:
                 if mod.type == 'ARMATURE' and mod.object == target_armature:
                      existing_arm_mod = mod
                      break
            if existing_arm_mod:
                 self.report({'INFO'}, f"Found existing Armature modifier '{existing_arm_mod.name}' targeting '{target_armature.name}'. Removing before adding new...") # Translated report
                 try:
                      source_mesh.modifiers.remove(existing_arm_mod)
                 except Exception as e:
                      self.report({'WARNING'}, f"Could not remove existing Armature mod '{existing_arm_mod.name}': {e}") # Translated report

            # --- 3. Add Armature Modifier ---
            self.report({'INFO'}, f"Adding Armature modifier '{self.armature_modifier_name}'...") # Translated report
            try:
                arm_mod = source_mesh.modifiers.new(name=self.armature_modifier_name, type='ARMATURE')
                arm_mod.object = target_armature
                arm_mod.use_vertex_groups = True # Crucial! Assumes VGroups exist from Step 2 or manually
                added_armature = True
                self.report({'INFO'}, f"Armature modifier '{arm_mod.name}' added, target '{target_armature.name}'.") # Translated report
            except Exception as e:
                self.report({'ERROR'}, f"Failed to add Armature modifier: {e}") # Translated report

        except Exception as e:
            self.report({'ERROR'}, f"Unexpected error during finalization: {e}") # Translated report
            # Fall through to finally for state restoration
        finally:
            # --- Restore State ---
            try:
                if original_mode is not None and source_mesh.name in context.view_layer.objects and source_mesh.mode != original_mode:
                    active_before = context.view_layer.objects.active
                    context.view_layer.objects.active = source_mesh
                    bpy.ops.object.mode_set(mode=original_mode)
                    context.view_layer.objects.active = active_before # Restore active
                elif context.object and context.object.mode != 'OBJECT':
                    bpy.ops.object.mode_set(mode='OBJECT') # Ensure object mode at least

                bpy.ops.object.select_all(action='DESELECT'); restored_active_flag = False
                for obj in original_selection:
                    if obj and obj.name in context.view_layer.objects:
                        obj.select_set(True)
                        if obj == original_active: context.view_layer.objects.active = obj; restored_active_flag = True
                if not restored_active_flag and original_active and original_active.name in context.view_layer.objects: context.view_layer.objects.active = original_active
                elif not context.view_layer.objects.active:
                    if source_mesh and source_mesh.name in context.view_layer.objects: context.view_layer.objects.active = source_mesh
                    elif original_active and original_active.name in context.view_layer.objects: context.view_layer.objects.active = original_active
            except Exception as e: print(f"Warning: Error during Finalize state restoration: {e}") # Kept print message

        # Report overall status
        final_status = f"Step 6 Complete. SD Removed: {'Yes' if removed_sd else 'No/Not Found'}. Armature Added: {'Yes' if added_armature else 'Error'}." # Translated report
        if added_armature:
             self.report({'INFO'}, final_status)
             return {'FINISHED'}
        else:
             self.report({'ERROR'}, final_status)
             return {'CANCELLED'}



# --- UI Panel (Updated for Step 6) ---
class RIG_PT_MeshRigPanel(bpy.types.Panel):
    bl_label = "Rig Mesh Plus"
    bl_idname = "RIG_PT_MeshRigPanel"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'Rig Tool' # Category name kept simple

    def draw(self, context):
        layout = self.layout
        scene = context.scene

        # --- Section 1: Create Rig ---
        box_create = layout.box(); col = box_create.column()
        row = col.row(align=True); row.alignment = 'LEFT'; row.label(text="1. Create Rig", icon='ARMATURE_DATA') # Translated label
        settings = col.column(align=True)
        settings.prop(scene, "rig_tool_bone_count")
        settings.prop(scene, "rig_tool_proximity_threshold")
        # Add Bone Length setting for the operator
        op = col.operator(RIG_OT_CreateMeshRig.bl_idname, text="Create Rig", icon='PLUS') # Translated text
        op.bone_length = scene.rig_tool_fixed_bone_length # Pass scene prop to operator prop
        settings.prop(scene, "rig_tool_fixed_bone_length") # Allow user to set it in UI

        info = col.row()
        mesh_sel = get_selected_mesh_object()
        if mesh_sel: info.label(text=f"Source: {mesh_sel.name}", icon='MESH_DATA') # Translated label
        else: info.label(text="Select a MESH object", icon='ERROR') # Translated label
        layout.separator()

        # --- Section 1.5: Merge Bones ---
        box_merge = layout.box(); col = box_merge.column()
        row = col.row(align=True); row.alignment = 'LEFT'; row.label(text="1.5 Merge Bones (Edit Mode)", icon='MOD_DECIM') # Translated label
        settings = col.column(align=True)
        settings.prop(scene, "rig_tool_merge_threshold_factor", text="Factor") # Translated text
        settings.prop(scene, "rig_tool_merge_min_count", text="Min Count") # Translated text
        col.separator()
        col.operator(RIG_OT_MergeSelectedBones.bl_idname, text="Merge Selected", icon='AUTOMERGE_ON') # Translated text
        info = col.row()
        is_arm_edit = (context.object and context.object.type == 'ARMATURE' and context.mode == 'EDIT_ARMATURE')
        sel_bones = context.selected_editable_bones
        min_count = scene.rig_tool_merge_min_count
        if not is_arm_edit: info.label(text="Requires Armature Edit Mode", icon='INFO') # Translated label
        elif not sel_bones or len(sel_bones) < min_count: info.label(text=f"Select >= {min_count} bones", icon='INFO') # Translated label
        else: info.label(text=f"Selected: {len(sel_bones)}", icon='CHECKMARK') # Translated label
        layout.separator()

        # --- Section 2: Parent Mesh ---
        box_parent = layout.box(); col = box_parent.column()
        row = col.row(align=True); row.alignment = 'LEFT'; row.label(text="2. Parent Mesh to Rig", icon='LINKED') # Translated label
        settings = col.column(align=True)
        settings.prop(scene, "rig_tool_parenting_mesh_target", text="Mesh") # Translated text
        settings.prop(scene, "rig_tool_parenting_armature_target", text="Armature") # Translated text
        col.separator()
        col.operator(RIG_OT_ParentMeshToRigTool.bl_idname, text="Parent Selected", icon='LINKED') # Translated text
        # Info rows condensed for brevity
        mesh_p = scene.rig_tool_parenting_mesh_target
        arm_p = scene.rig_tool_parenting_armature_target
        col.label(text=f"Mesh: {mesh_p.name if mesh_p else 'None'}", icon='MESH_DATA' if mesh_p else 'QUESTION') # Translated label
        col.label(text=f"Arm: {arm_p.name if arm_p else 'None'}", icon='ARMATURE_DATA' if arm_p else 'QUESTION') # Translated label
        layout.separator()

        # --- Section 3: Constraints -> Pose -> Remove Mod ---
        box_con = layout.box(); col = box_con.column()
        row = col.row(align=True); row.alignment = 'LEFT'; row.label(text="3. Constr -> Pose -> Del Mod", icon='CONSTRAINT_BONE') # Translated label
        settings = col.column(align=True)
        settings.prop(scene, "rig_tool_constraint_target", text="Target Mesh (VG)") # Translated text
        col.separator()
        col.operator(RIG_OT_AddBoneConstraints.bl_idname, text="Execute Step 3", icon='HOOK') # Translated text
        arm_sel = get_selected_armature_object()
        mesh_vg = scene.rig_tool_constraint_target
        mesh_mod = scene.rig_tool_parenting_mesh_target # Mesh for mod removal info
        col.label(text=f"Arm: {arm_sel.name if arm_sel else 'Select!'}", icon='ARMATURE_DATA' if arm_sel else 'ERROR') # Translated label
        col.label(text=f"VG Target: {mesh_vg.name if mesh_vg else 'None'}", icon='MESH_DATA' if mesh_vg else 'QUESTION') # Translated label
        col.label(text=f"Del Mod from: {mesh_mod.name if mesh_mod else 'None'}", icon='MODIFIER_OFF' if mesh_mod else 'INFO') # Translated label
        layout.separator()

        # --- Section 4: Surface Deform ---
        box_sd = layout.box(); col = box_sd.column()
        row = col.row(align=True); row.alignment = 'LEFT'; row.label(text="4. Surface Deform", icon='MOD_MESHDEFORM') # Translated label
        settings = col.column(align=True)
        settings.prop(scene, "rig_tool_surfdeform_source_mesh", text="Source") # Translated text
        settings.prop(scene, "rig_tool_surfdeform_target_mesh", text="Target (Bind)") # Translated text
        col.separator()
        col.operator(RIG_OT_AddSurfaceDeform.bl_idname, text="Add/Update & Bind", icon='MOD_MESHDEFORM') # Translated text
        mesh_sd_s = scene.rig_tool_surfdeform_source_mesh
        mesh_sd_t = scene.rig_tool_surfdeform_target_mesh
        col.label(text=f"Source: {mesh_sd_s.name if mesh_sd_s else 'None'}", icon='MESH_DATA' if mesh_sd_s else 'QUESTION') # Translated label
        col.label(text=f"Target: {mesh_sd_t.name if mesh_sd_t else 'None'}", icon='MESH_DATA' if mesh_sd_t else 'QUESTION') # Translated label
        layout.separator()

        # --- Section 5: Bake Action ---
        box_bake = layout.box(); col = box_bake.column()
        row = col.row(align=True); row.alignment = 'LEFT'; row.label(text="5. Bake Action", icon='ACTION') # Translated label
        settings = col.column(align=True)
        settings.prop(scene, "rig_tool_bake_armature_target", text="Armature") # Translated text
        row_frames = settings.row(align=True)
        row_frames.prop(scene, "rig_tool_bake_start_frame", text="S") # Translated text (Start)
        row_frames.prop(scene, "rig_tool_bake_end_frame", text="E") # Translated text (End)
        # Info about fixed options can be in tooltip or description
        col.separator()
        col.operator(RIG_OT_BakeRigAction.bl_idname, text="Bake Selected Armature", icon='RENDER_ANIMATION') # Translated text
        arm_bake = scene.rig_tool_bake_armature_target
        col.label(text=f"Arm: {arm_bake.name if arm_bake else 'None'}", icon='ARMATURE_DATA' if arm_bake else 'QUESTION') # Translated label
        layout.separator()

        # --- Section 6: Finalize Deform ---
        box_fin = layout.box(); col = box_fin.column()
        row = col.row(align=True); row.alignment = 'LEFT'; row.label(text="6. Finalize Deform", icon='MOD_ARMATURE') # Translated label
        settings = col.column(align=True)
        settings.prop(scene, "rig_tool_final_source_mesh", text="Source Mesh") # Translated text
        settings.prop(scene, "rig_tool_final_target_armature", text="Target Armature") # Translated text
        col.separator()
        col.operator(RIG_OT_FinalizeDeform.bl_idname, text="Remove SD, Add Armature", icon='CHECKMARK') # Translated text
        mesh_fin_s = scene.rig_tool_final_source_mesh
        arm_fin_t = scene.rig_tool_final_target_armature
        col.label(text=f"Source: {mesh_fin_s.name if mesh_fin_s else 'None'}", icon='MESH_DATA' if mesh_fin_s else 'QUESTION') # Translated label
        col.label(text=f"Target: {arm_fin_t.name if arm_fin_t else 'None'}", icon='ARMATURE_DATA' if arm_fin_t else 'QUESTION') # Translated label

# --- Registration ---

classes = (
    RIG_OT_CreateMeshRig,
    RIG_OT_MergeSelectedBones,
    RIG_OT_ParentMeshToRigTool,
    RIG_OT_AddBoneConstraints,
    RIG_OT_AddSurfaceDeform,
    RIG_OT_BakeRigAction,
    RIG_OT_FinalizeDeform, # Added Step 6 Operator
    RIG_PT_MeshRigPanel,
)

def register():
    # --- Properties Definitions ---
    bpy.types.Scene.rig_tool_bone_count = bpy.props.IntProperty(name="Bone Count", default=100, min=1) # Translated name
    bpy.types.Scene.rig_tool_proximity_threshold = bpy.props.FloatProperty(name="Proximity Threshold", default=0.03, min=0.0, subtype='DISTANCE', unit='LENGTH', precision=4) # Translated name
    # NEW: Scene property for fixed bone length used by Create Rig operator
    bpy.types.Scene.rig_tool_fixed_bone_length = bpy.props.FloatProperty(name="Bone Length (Creation)", description="Fixed length for bones during creation", default=0.005, min=0.0001, max=1.0, subtype='DISTANCE', unit='LENGTH', precision=4) # Translated name & description

    bpy.types.Scene.rig_tool_merge_threshold_factor = bpy.props.FloatProperty(name="Merge Radius Factor", default=2.5, min=1.01, precision=2) # Translated name
    bpy.types.Scene.rig_tool_merge_min_count = bpy.props.IntProperty(name="Min Bones to Merge", default=3, min=2) # Translated name
    bpy.types.Scene.rig_tool_parenting_mesh_target = bpy.props.PointerProperty(type=bpy.types.Object, name="Mesh for Parenting", poll=lambda self, obj: obj.type == 'MESH') # Translated name
    bpy.types.Scene.rig_tool_parenting_armature_target = bpy.props.PointerProperty(type=bpy.types.Object, name="Armature for Parenting", poll=lambda self, obj: obj.type == 'ARMATURE') # Translated name
    bpy.types.Scene.rig_tool_constraint_target = bpy.props.PointerProperty(type=bpy.types.Object, name="Constraint Target (VG)", poll=lambda self, obj: obj.type == 'MESH') # Translated name
    bpy.types.Scene.rig_tool_surfdeform_source_mesh = bpy.props.PointerProperty(type=bpy.types.Object, name="Source Mesh (SD)", poll=lambda self, obj: obj.type == 'MESH') # Translated name
    bpy.types.Scene.rig_tool_surfdeform_target_mesh = bpy.props.PointerProperty(type=bpy.types.Object, name="Target Mesh (SD Bind)", poll=lambda self, obj: obj.type == 'MESH') # Translated name
    bpy.types.Scene.rig_tool_bake_armature_target = bpy.props.PointerProperty(type=bpy.types.Object, name="Armature for Baking", poll=lambda self, obj: obj.type == 'ARMATURE') # Translated name
    bpy.types.Scene.rig_tool_bake_start_frame = bpy.props.IntProperty(name="Start Frame", default=1) # Translated name
    bpy.types.Scene.rig_tool_bake_end_frame = bpy.props.IntProperty(name="End Frame", default=250) # Translated name
    # NEW: Properties for Step 6
    bpy.types.Scene.rig_tool_final_source_mesh = bpy.props.PointerProperty(type=bpy.types.Object, name="Source Mesh (Final)", description="Mesh to remove SD from and add Armature mod to", poll=lambda self, obj: obj.type == 'MESH') # Translated name & description
    bpy.types.Scene.rig_tool_final_target_armature = bpy.props.PointerProperty(type=bpy.types.Object, name="Target Armature (Final)", description="Armature for the final Armature mod", poll=lambda self, obj: obj.type == 'ARMATURE') # Translated name & description

    # --- Register Classes ---
    for cls in classes:
        bpy.utils.register_class(cls)

    # --- Set Initial Bake Frame Range ---
    try:
         current_scene = bpy.context.scene
         current_scene.rig_tool_bake_start_frame = current_scene.frame_start
         current_scene.rig_tool_bake_end_frame = current_scene.frame_end
    except Exception: pass # Ignore errors if scene frames aren't set

    v = bl_info.get('version', (0, 0, 0))
    print(f"Rig Mesh Plus (v{v[0]}.{v[1]}.{v[2]}) Registered") # Kept English message


def unregister():
     props_to_delete = [
         "rig_tool_bone_count", "rig_tool_proximity_threshold", "rig_tool_fixed_bone_length", # Added fixed_bone_length
         "rig_tool_merge_threshold_factor", "rig_tool_merge_min_count",
         "rig_tool_parenting_mesh_target", "rig_tool_parenting_armature_target",
         "rig_tool_constraint_target",
         "rig_tool_surfdeform_source_mesh", "rig_tool_surfdeform_target_mesh",
         "rig_tool_bake_armature_target", "rig_tool_bake_start_frame", "rig_tool_bake_end_frame",
         "rig_tool_final_source_mesh", "rig_tool_final_target_armature", # Added Step 6 props
     ]
     for prop_name in props_to_delete:
         if hasattr(bpy.types.Scene, prop_name):
             try: delattr(bpy.types.Scene, prop_name)
             except Exception as e: print(f"Warn: Could not delete prop '{prop_name}': {e}") # Kept English message

     for cls in reversed(classes):
         try: bpy.utils.unregister_class(cls)
         except RuntimeError: pass # Might already be unregistered

     print("Rig Mesh Plus Unregistered") # Kept English message

if __name__ == "__main__":
    # Reregister for testing
    try: unregister()
    except Exception: pass
    register()